@@ -1,5 +1,5 @@
 package HackaMol::Roles::ReadPdbRole;
-$HackaMol::Roles::ReadPdbRole::VERSION = '0.047';
+$HackaMol::Roles::ReadPdbRole::VERSION = '0.048';
 # ABSTRACT: Read files with molecular information
 use Moose::Role;
 use HackaMol::PeriodicTable qw(_element_name _trim _qstring_num);
@@ -91,7 +91,7 @@ sub read_pdb_atoms {
             if   ( $charge =~ m/\d/ ) { $charge = _qstring_num($charge) }
             else                      { $charge = 0 }
 
-            if   ( $chainID =~ m/\w/ ) { $chainID = uc( _trim($chainID) ) }
+            if   ( $chainID =~ m/\w/ ) { $chainID =  _trim($chainID) }
             else                       { $chainID = ' ' }
 
             $name    = _trim($name);
@@ -184,7 +184,7 @@ HackaMol::Roles::ReadPdbRole - Read files with molecular information
 
 =head1 VERSION
 
-version 0.047
+version 0.048
 
 =head1 SYNOPSIS
 
